@@ -1011,23 +1011,25 @@ srl_read_objectv(pTHX_ srl_decoder_t *dec, SV* into, U8 obj_tag)
     if (expect_false( SRL_DEC_HAVE_OPTION(dec, SRL_F_DECODER_REFUSE_OBJECTS) ))
         SRL_ERROR_REFUSE_OBJECT();
 
-    ofs= srl_read_varint_uv_offset(aTHX_ dec," while reading OBJECTV classname");
+    ofs= srl_read_varint_uv_offset(aTHX_ dec," while reading OBJECTV(_FREEZE) classname");
 
     if (expect_false( !dec->ref_bless_av ))
-        SRL_ERROR("Corrupted packet. OBJECTV used without preceding OBJECT to define classname");
+        SRL_ERROR("Corrupted packet. OBJECTV(_FREEZE) used without "
+                  "preceding OBJECT(_FREEZE) to define classname");
     av= (AV *)PTABLE_fetch(dec->ref_bless_av, (void *)ofs);
     if (expect_false( NULL == av )) {
-        SRL_ERRORf1("Corrupted packet. OBJECTV references unknown classname offset: %lu", (unsigned long)ofs);
+        SRL_ERRORf1("Corrupted packet. OBJECTV(_FREEZE) references unknown classname offset: %lu", (unsigned long)ofs);
     }
 
-
-    if (expect_false( obj_tag == SRL_HDR_OBJECT_FREEZE )) {
+    /* checking tag: SRL_HDR_OBJECTV_FREEZE or SRL_HDR_OBJECTV? */
+    if (expect_false( obj_tag == SRL_HDR_OBJECTV_FREEZE )) {
         HV *class_stash= PTABLE_fetch(dec->ref_stashes, (void *)ofs);
         if (expect_false( class_stash == NULL ))
-            SRL_ERROR("Corrupted packet. OBJECTV used without preceding OBJECT to define classname");
+            SRL_ERROR("Corrupted packet. OBJECTV(_FREEZE) used without "
+                      "preceding OBJECT(_FREEZE) to define classname");
         srl_read_frozen_object(aTHX_ dec, class_stash, into);
     }  else {
-
+        /* SRL_HDR_OBJECTV, not SRL_HDR_OBJECTV_FREEZE */
         /* now deparse the thing we are going to bless */
         srl_read_single_value(aTHX_ dec, into);
 
@@ -1039,7 +1041,8 @@ srl_read_objectv(pTHX_ srl_decoder_t *dec, SV* into, U8 obj_tag)
             /* See 'define USE_588_WORKAROUND' above for a discussion of what this does. */
             HV *class_stash= PTABLE_fetch(dec->ref_stashes, (void *)ofs);
             if (expect_false( class_stash == NULL ))
-                SRL_ERROR("Corrupted packet. OBJECTV used without preceding OBJECT to define classname");
+                SRL_ERROR("Corrupted packet. OBJECTV(_FREEZE) used without "
+                          "preceding OBJECT(_FREEZE) to define classname");
             if (!SRL_DEC_HAVE_OPTION(dec, SRL_F_DECODER_NO_BLESS_OBJECTS))
                 sv_bless(into, class_stash);
         }
@@ -1094,9 +1097,16 @@ srl_read_object(pTHX_ srl_decoder_t *dec, SV* into, U8 obj_tag)
     if (tag == SRL_HDR_COPY) {
         ofs= srl_read_varint_uv_offset(aTHX_ dec, " while reading COPY class name");
         storepos= ofs;
-        if (expect_true( dec->ref_seenhash != NULL )) {
-            class_stash= PTABLE_fetch(dec->ref_seenhash, (void *)ofs);
+        /* if this string was seen before as part of a classname then we expect
+         * a stash available below. However it might have been serialized as a key
+         * or something like that, which would mean we dont have an entry in ref_stashes
+         * anymore. So first we check if we have a stash. If we do, then we can avoid
+         * some work. */
+        if (expect_true( dec->ref_stashes != NULL )) {
+            class_stash= PTABLE_fetch(dec->ref_stashes, (void *)ofs);
         }
+        /* Check if we actually got a class_stash back. If we didn't then we need
+         * to deserialize the class name */
         if (!class_stash) {
             from= dec->body_pos + ofs;
             tag= *from++;
@@ -1118,28 +1128,33 @@ srl_read_object(pTHX_ srl_decoder_t *dec, SV* into, U8 obj_tag)
                 SRL_ERROR_BAD_COPY(dec, SRL_HDR_OBJECT);
             }
         }
-        /* NOTREACHED */
     } else {
         SRL_ERROR_UNEXPECTED(dec,tag, "a class name");
     }
 
+    /* At this point we may or may not have a class stash. If they used a Copy there
+     * is a decent chance we do. */
     SRL_ASSERT_REF_PTR_TABLES(dec);
     if (!class_stash) {
+        /* no class stash - so we need to look it up and then store it away for future use */
         class_stash= gv_stashpvn((char *)from, key_len, flags);
         PTABLE_store(dec->ref_stashes, (void *)storepos, (void *)class_stash);
+        /* Since this is the first time we have seen this stash then it is the first time
+         * that we have stored an item in the ref_bless_av hash as well. So create a new one
+         * and store it away. */
         av= newAV();
         sv_2mortal((SV*)av);
         PTABLE_store(dec->ref_bless_av, (void *)storepos, (void *)av);
     } else {
-        if (NULL == (av= (AV *)PTABLE_fetch(dec->ref_bless_av, (void *)storepos)) )
+        /* we have a class stash so we should have a ref_bless_av as well. */
+        av= (AV *)PTABLE_fetch(dec->ref_bless_av, (void *)storepos);
+        if ( !av )
             SRL_ERRORf1("Panic, no ref_bless_av for %lu", (unsigned long)storepos);
     }
 
     if (expect_false( obj_tag == SRL_HDR_OBJECT_FREEZE )) {
         srl_read_frozen_object(aTHX_ dec, class_stash, into);
     }  else {
-
-
         /* We now have a stash so we /could/ bless... except that
          * we don't actually want to do so right now. We want to defer blessing
          * until the full packet has been read. Yes it is more overhead, but
@@ -1171,7 +1186,7 @@ srl_read_frozen_object(pTHX_ srl_decoder_t *dec, HV *class_stash, SV *into)
     char *classname = HvNAME(class_stash);
     SV* referent;
     SV *replacement;
-    
+
     /* At this point in the input stream we should have REFN WHATEVER. The WHATEVER
      * may be referenced from multiple RV's in the data structure, which means that
      * srl_read_single_value() will cache the *unthawed* representation when we finally
@@ -1186,9 +1201,17 @@ srl_read_frozen_object(pTHX_ srl_decoder_t *dec, HV *class_stash, SV *into)
 
     srl_read_single_value(aTHX_ dec, into);
 
+    /* Assert that we got a top level array ref as the spec requires.
+     * Not throwing an exception here violates expectations down the line and
+     * can lead to segfaults. */
+    if (expect_false( !SvROK(into) || SvTYPE(SvRV(into)) != SVt_PVAV ))
+        SRL_ERROR("Corrupted packet. OBJECT(V)_FREEZE used without "
+                  "being followed by an array reference");
+
     {
         int count;
         AV *arg_av= (AV*)SvRV(into);
+        int arg_av_len = av_len(arg_av)+1;
         dSP;
 
         ENTER;
@@ -1201,8 +1224,8 @@ srl_read_frozen_object(pTHX_ srl_decoder_t *dec, HV *class_stash, SV *into)
         /* FIXME do not recreate the following SV. That's dumb and wasteful! - so long as it doesnt get modified! */
         PUSHs(sv_2mortal(newSVpvs("Sereal")));
         /* Push the args into the stack */
-        for (count=0 ; count < av_len(arg_av) + 1; count++) {
-            PUSHs((SV*)*av_fetch(arg_av,count,0));
+        for (count=0 ; count < arg_av_len; count++) {
+            PUSHs((SV*)*av_fetch(arg_av, count, 0));
         }
 
         PUTBACK;
@@ -1222,6 +1245,7 @@ srl_read_frozen_object(pTHX_ srl_decoder_t *dec, HV *class_stash, SV *into)
         FREETMPS;
         LEAVE;
     }
+
     /* At this point "into" is an SvRV pointing at the *unthawed* representation.
      * This means we need to a) remove the old unthawed item and dispose of it
      * and b) make "into" point at the replacement, and c) if necessary store the
